@@ -131,7 +131,6 @@ void ProcessGpsInfo(unsigned char * DirStart, int ByteCountUnused, unsigned char
                     ErrNonfatal("Inappropriate format (%d) for GPS coordinates!", Format, 0);
                 }
                 strcpy(FmtString, "%0.0fd %0.0fm %0.0fs");
-
                 for (a=0;a<3;a++){
                     int den, digits;
 
@@ -146,6 +145,7 @@ void ProcessGpsInfo(unsigned char * DirStart, int ByteCountUnused, unsigned char
 
                     Values[a] = ConvertAnyFormat(ValuePtr+a*ComponentSize, Format);
                 }
+
                 sprintf(TempString, FmtString, Values[0], Values[1], Values[2]);
 
                 if (Tag == TAG_GPS_LAT){
@@ -160,7 +160,8 @@ void ProcessGpsInfo(unsigned char * DirStart, int ByteCountUnused, unsigned char
                 break;
 
             case TAG_GPS_ALT:
-                sprintf(ImageInfo.GpsAlt + 1, "%dm", Get32s(ValuePtr));
+                sprintf(ImageInfo.GpsAlt + 1, "%.2fm", 
+                    ConvertAnyFormat(ValuePtr, Format));
                 break;
         }
 
@@ -172,7 +173,7 @@ void ProcessGpsInfo(unsigned char * DirStart, int ByteCountUnused, unsigned char
                 // Show unknown tag
                 printf("        Illegal GPS tag %04x=", Tag);
             }
-    
+
             switch(Format){
                 case FMT_UNDEFINED:
                     // Undefined is typically an ascii string.
